@@ -357,7 +357,7 @@ def ConvNeXt(
             won't be used.
         default_size: Default input image size.
         name: An optional name for the model.
-        include_preprocessing: boolean denoting whther to
+        include_preprocessing: boolean denoting whether to
             include preprocessing in the model.
             When `weights="imagenet"` this should always be `True`.
             But for other models (e.g., randomly initialized) you should set it
@@ -432,10 +432,11 @@ def ConvNeXt(
 
     if input_tensor is not None:
         inputs = operation_utils.get_source_inputs(input_tensor)[0]
+        x = input_tensor
     else:
         inputs = img_input
+        x = inputs
 
-    x = inputs
     if include_preprocessing:
         channel_axis = (
             3 if backend.image_data_format() == "channels_last" else 1
@@ -522,6 +523,30 @@ def ConvNeXt(
 
     model = Functional(inputs=inputs, outputs=x, name=name)
 
+    # Validate weights before requesting them from the API
+    if weights == "imagenet":
+        expected_config = MODEL_CONFIGS[weights_name.split("convnext_")[-1]]
+        if (
+            depths != expected_config["depths"]
+            or projection_dims != expected_config["projection_dims"]
+        ):
+            raise ValueError(
+                f"Architecture configuration does not match {weights_name} "
+                f"variant. When using pre-trained weights, the model "
+                f"architecture must match the pre-trained configuration "
+                f"exactly. Expected depths: {expected_config['depths']}, "
+                f"got: {depths}. Expected projection_dims: "
+                f"{expected_config['projection_dims']}, got: {projection_dims}."
+            )
+
+        if weights_name not in name:
+            raise ValueError(
+                f'Model name "{name}" does not match weights variant '
+                f'"{weights_name}". When using imagenet weights, model name '
+                f'must contain the weights variant (e.g., "convnext_'
+                f'{weights_name.split("convnext_")[-1]}").'
+            )
+
     # Load weights.
     if weights == "imagenet":
         if include_top:
