@@ -574,7 +574,7 @@ Args:
         - `max` means that global max pooling will be applied.
     classes: optional number of classes to classify images into, only to be
         specified if `include_top` is `True`, and if no `weights` argument is
-        specified.
+        specified. Defaults to `1000`.
     classifier_activation: A `str` or callable. The activation function to
         use on the "top" layer. Ignored unless `include_top=True`. Set
         `classifier_activation=None` to return the logits of the "top" layer.
