@@ -1,9 +1,9 @@
-import jax
 import ml_dtypes
 import numpy as np
 
 from keras.src import backend
 from keras.src.backend.numpy.core import convert_to_tensor
+from keras.src.random.seed_generator import draw_seed
 from keras.src.utils.module_utils import scipy
 
 RESIZE_INTERPOLATIONS = (
@@ -364,12 +364,145 @@ def resize(
                 padded_img = images
         images = padded_img
 
-    return np.array(
-        jax.image.resize(
-            images, size, method=interpolation, antialias=antialias
-        )
+    return _resize(images, size, method=interpolation, antialias=antialias)
+
+
+def compute_weight_mat(
+    input_size, output_size, scale, translation, kernel, antialias
+):
+    dtype = np.result_type(scale, translation)
+    inv_scale = 1.0 / scale
+    kernel_scale = np.maximum(inv_scale, 1.0) if antialias else 1.0
+
+    sample_f = (
+        (np.arange(output_size, dtype=dtype) + 0.5) * inv_scale
+        - translation * inv_scale
+        - 0.5
     )
 
+    x = (
+        np.abs(
+            sample_f[np.newaxis, :]
+            - np.arange(input_size, dtype=dtype)[:, np.newaxis]
+        )
+        / kernel_scale
+    )
+
+    weights = kernel(x)
+
+    total_weight_sum = np.sum(weights, axis=0, keepdims=True)
+    weights = np.where(
+        np.abs(total_weight_sum) > 1000.0 * np.finfo(np.float32).eps,
+        np.divide(
+            weights, np.where(total_weight_sum != 0, total_weight_sum, 1)
+        ),
+        0,
+    )
+
+    input_size_minus_0_5 = input_size - 0.5
+    return np.where(
+        np.logical_and(sample_f >= -0.5, sample_f <= input_size_minus_0_5)[
+            np.newaxis, :
+        ],
+        weights,
+        0,
+    )
+
+
+def _resize(image, shape, method, antialias):
+    def _fill_triangle_kernel(x):
+        return np.maximum(0, 1 - np.abs(x))
+
+    def _fill_keys_cubic_kernel(x):
+        out = ((1.5 * x - 2.5) * x) * x + 1.0
+        out = np.where(x >= 1.0, ((-0.5 * x + 2.5) * x - 4.0) * x + 2.0, out)
+        return np.where(x >= 2.0, 0.0, out)
+
+    def _fill_lanczos_kernel(radius, x):
+        y = radius * np.sin(np.pi * x) * np.sin(np.pi * x / radius)
+        out = np.where(
+            x > 1e-3, np.divide(y, np.where(x != 0, np.pi**2 * x**2, 1)), 1
+        )
+        return np.where(x > radius, 0.0, out)
+
+    if method == "nearest":
+        return _resize_nearest(image, shape)
+    elif method == "bilinear":
+        kernel = _fill_triangle_kernel
+    elif method == "lanczos3":
+        kernel = lambda x: _fill_lanczos_kernel(3.0, x)
+    elif method == "lanczos5":
+        kernel = lambda x: _fill_lanczos_kernel(5.0, x)
+    elif method == "bicubic":
+        kernel = _fill_keys_cubic_kernel
+    else:
+        raise ValueError("Unknown resize method")
+
+    spatial_dims = tuple(
+        i for i in range(len(shape)) if image.shape[i] != shape[i]
+    )
+    scale = [
+        shape[d] / image.shape[d] if image.shape[d] != 0 else 1.0
+        for d in spatial_dims
+    ]
+
+    return _scale_and_translate(
+        image,
+        shape,
+        spatial_dims,
+        scale,
+        [0.0] * len(spatial_dims),
+        kernel,
+        antialias,
+    )
+
+
+def _resize_nearest(x, output_shape):
+    input_shape = x.shape
+    spatial_dims = tuple(
+        i for i in range(len(input_shape)) if input_shape[i] != output_shape[i]
+    )
+
+    for d in spatial_dims:
+        m, n = input_shape[d], output_shape[d]
+        offsets = (np.arange(n, dtype=np.float32) + 0.5) * m / n
+        offsets = np.floor(offsets).astype(np.int32)
+        indices = [slice(None)] * len(input_shape)
+        indices[d] = offsets
+        x = x[tuple(indices)]
+    return x
+
+
+def _scale_and_translate(
+    x, output_shape, spatial_dims, scale, translation, kernel, antialias
+):
+    input_shape = x.shape
+
+    if len(spatial_dims) == 0:
+        return x
+
+    if np.issubdtype(x.dtype, np.integer):
+        output = x.astype(np.float32)
+        use_rounding = True
+    else:
+        output = x.copy()
+        use_rounding = False
+
+    for i, d in enumerate(spatial_dims):
+        d = d % x.ndim
+        m, n = input_shape[d], output_shape[d]
+
+        w = compute_weight_mat(
+            m, n, scale[i], translation[i], kernel, antialias
+        ).astype(np.float32)
+        output = np.tensordot(output, w, axes=(d, 0))
+        output = np.moveaxis(output, -1, d)
+
+    if use_rounding:
+        output = np.clip(np.round(output), x.min(), x.max())
+        output = output.astype(x.dtype)
+    return output
+
 
 AFFINE_TRANSFORM_INTERPOLATIONS = {  # map to order
     "nearest": 0,
@@ -467,7 +600,7 @@ def affine_transform(
     # transform the indices
     coordinates = np.einsum("Bhwij, Bjk -> Bhwik", indices, transform)
     coordinates = np.moveaxis(coordinates, source=-1, destination=1)
-    coordinates += np.reshape(a=offset, newshape=(*offset.shape, 1, 1, 1))
+    coordinates += np.reshape(offset, newshape=(*offset.shape, 1, 1, 1))
 
     # apply affine transformation
     affined = np.stack(
@@ -878,3 +1011,127 @@ def gaussian_blur(
         blurred_images = np.squeeze(blurred_images, axis=0)
 
     return blurred_images
+
+
+def elastic_transform(
+    images,
+    alpha=20.0,
+    sigma=5.0,
+    interpolation="bilinear",
+    fill_mode="reflect",
+    fill_value=0.0,
+    seed=None,
+    data_format=None,
+):
+    data_format = backend.standardize_data_format(data_format)
+    if interpolation not in AFFINE_TRANSFORM_INTERPOLATIONS.keys():
+        raise ValueError(
+            "Invalid value for argument `interpolation`. Expected of one "
+            f"{set(AFFINE_TRANSFORM_INTERPOLATIONS.keys())}. Received: "
+            f"interpolation={interpolation}"
+        )
+    if fill_mode not in AFFINE_TRANSFORM_FILL_MODES:
+        raise ValueError(
+            "Invalid value for argument `fill_mode`. Expected of one "
+            f"{AFFINE_TRANSFORM_FILL_MODES}. Received: fill_mode={fill_mode}"
+        )
+    if len(images.shape) not in (3, 4):
+        raise ValueError(
+            "Invalid images rank: expected rank 3 (single image) "
+            "or rank 4 (batch of images). Received input with shape: "
+            f"images.shape={images.shape}"
+        )
+
+    images = convert_to_tensor(images)
+    input_dtype = images.dtype
+
+    alpha = convert_to_tensor(alpha, dtype=input_dtype)
+    sigma = convert_to_tensor(sigma, dtype=input_dtype)
+
+    kernel_size = (int(6 * sigma) | 1, int(6 * sigma) | 1)
+
+    need_squeeze = False
+    if len(images.shape) == 3:
+        images = np.expand_dims(images, axis=0)
+        need_squeeze = True
+
+    if data_format == "channels_last":
+        batch_size, height, width, channels = images.shape
+        channel_axis = -1
+    else:
+        batch_size, channels, height, width = images.shape
+        channel_axis = 1
+
+    seed = draw_seed(seed)
+    rng = np.random.default_rng(seed)
+    dx = (
+        rng.normal(size=(batch_size, height, width), loc=0.0, scale=1.0).astype(
+            input_dtype
+        )
+        * sigma
+    )
+    dy = (
+        rng.normal(size=(batch_size, height, width), loc=0.0, scale=1.0).astype(
+            input_dtype
+        )
+        * sigma
+    )
+
+    dx = gaussian_blur(
+        np.expand_dims(dx, axis=channel_axis),
+        kernel_size=kernel_size,
+        sigma=(sigma, sigma),
+        data_format=data_format,
+    )
+    dy = gaussian_blur(
+        np.expand_dims(dy, axis=channel_axis),
+        kernel_size=kernel_size,
+        sigma=(sigma, sigma),
+        data_format=data_format,
+    )
+
+    dx = np.squeeze(dx)
+    dy = np.squeeze(dy)
+
+    x, y = np.meshgrid(np.arange(width), np.arange(height))
+    x, y = x[None, :, :], y[None, :, :]
+
+    distorted_x = x + alpha * dx
+    distorted_y = y + alpha * dy
+
+    transformed_images = np.zeros_like(images)
+
+    if data_format == "channels_last":
+        for i in range(channels):
+            transformed_images[..., i] = np.stack(
+                [
+                    map_coordinates(
+                        images[b, ..., i],
+                        [distorted_y[b], distorted_x[b]],
+                        order=AFFINE_TRANSFORM_INTERPOLATIONS[interpolation],
+                        fill_mode=fill_mode,
+                        fill_value=fill_value,
+                    )
+                    for b in range(batch_size)
+                ]
+            )
+    else:
+        for i in range(channels):
+            transformed_images[:, i, :, :] = np.stack(
+                [
+                    map_coordinates(
+                        images[b, i, ...],
+                        [distorted_y[b], distorted_x[b]],
+                        order=AFFINE_TRANSFORM_INTERPOLATIONS[interpolation],
+                        fill_mode=fill_mode,
+                        fill_value=fill_value,
+                    )
+                    for b in range(batch_size)
+                ]
+            )
+
+    if need_squeeze:
+        transformed_images = np.squeeze(transformed_images, axis=0)
+    transformed_images = transformed_images.astype(input_dtype)
+
+    return transformed_images
