@@ -1,3 +1,4 @@
+import builtins
 import contextlib
 import warnings
 
@@ -109,6 +110,13 @@ def get_ov_output(x, ov_type=None):
             x = ov_opset.constant(x, OPENVINO_DTYPES["bfloat16"]).output(0)
         else:
             x = ov_opset.constant(x).output(0)
+    elif isinstance(x, (list, tuple)):
+        if isinstance(x, tuple):
+            x = list(x)
+        if ov_type is None:
+            x = ov_opset.constant(x).output(0)
+        else:
+            x = ov_opset.constant(x, ov_type).output(0)
     elif np.isscalar(x):
         x = ov_opset.constant(x).output(0)
     elif isinstance(x, KerasVariable):
@@ -187,6 +195,10 @@ class OpenVINOKerasTensor:
         first, other = align_operand_types(
             first, other, "OpenVINOKerasTensor::__mul__"
         )
+        if first.get_element_type() == Type.boolean:
+            return OpenVINOKerasTensor(
+                ov_opset.logical_and(first, other).output(0)
+            )
         return OpenVINOKerasTensor(ov_opset.multiply(first, other).output(0))
 
     def __rmul__(self, other):
@@ -195,6 +207,10 @@ class OpenVINOKerasTensor:
         first, other = align_operand_types(
             first, other, "OpenVINOKerasTensor::__rmul__"
         )
+        if first.get_element_type() == Type.boolean:
+            return OpenVINOKerasTensor(
+                ov_opset.logical_and(first, other).output(0)
+            )
         return OpenVINOKerasTensor(ov_opset.multiply(first, other).output(0))
 
     def __truediv__(self, other):
@@ -249,31 +265,222 @@ class OpenVINOKerasTensor:
         )
         return OpenVINOKerasTensor(ov_opset.power(first, other).output(0))
 
-    def __getitem__(self, indices):
-        # now it has limited functionaly
-        # and supports only a case with one integer index in indices
-        # other indices must be None
-        data = self.output
-        axis = []
-        gather_index = None
-        if isinstance(indices, int):
-            indices = (indices,)
-        assert isinstance(indices, tuple), "only tuple is supported"
-        for dim, index in enumerate(indices):
-            if isinstance(index, int):
-                axis.append(dim)
-                gather_index = ov_opset.constant(index, Type.i32)
-            else:
-                assert (
-                    index.start is None
-                    and index.stop is None
-                    and index.step is None
-                )
-        assert len(axis) == 1, "axis must contain one element"
-        axis = ov_opset.constant(axis, Type.i32)
-        return OpenVINOKerasTensor(
-            ov_opset.gather(data, gather_index, axis).output(0)
+    def __rpow__(self, other):
+        other = get_ov_output(other)
+        first = self.output
+        first, other = align_operand_types(
+            first, other, "OpenVINOKerasTensor::__rpow__"
         )
+        return OpenVINOKerasTensor(ov_opset.power(other, first).output(0))
+
+    def __lt__(self, other):
+        first = self.output
+        other = get_ov_output(other)
+        first, other = align_operand_types(
+            first, other, "OpenVINOKerasTensor::__lt__"
+        )
+        return OpenVINOKerasTensor(ov_opset.less(first, other).output(0))
+
+    def __gt__(self, other):
+        first = self.output
+        other = get_ov_output(other)
+        first, other = align_operand_types(
+            first, other, "OpenVINOKerasTensor::__gt__"
+        )
+        return OpenVINOKerasTensor(ov_opset.greater(first, other).output(0))
+
+    def __le__(self, other):
+        first = self.output
+        other = get_ov_output(other)
+        first, other = align_operand_types(
+            first, other, "OpenVINOKerasTensor::__le__"
+        )
+        return OpenVINOKerasTensor(ov_opset.less_equal(first, other).output(0))
+
+    def __ge__(self, other):
+        first = self.output
+        other = get_ov_output(other)
+        first, other = align_operand_types(
+            first, other, "OpenVINOKerasTensor::__ge__"
+        )
+        return OpenVINOKerasTensor(
+            ov_opset.greater_equal(first, other).output(0)
+        )
+
+    def __eq__(self, other):
+        first = self.output
+        other = get_ov_output(other)
+        first, other = align_operand_types(
+            first, other, "OpenVINOKerasTensor::__eq__"
+        )
+        return OpenVINOKerasTensor(ov_opset.equal(first, other).output(0))
+
+    def __ne__(self, other):
+        first = self.output
+        other = get_ov_output(other)
+        first, other = align_operand_types(
+            first, other, "OpenVINOKerasTensor::__ne__"
+        )
+        return OpenVINOKerasTensor(ov_opset.not_equal(first, other).output(0))
+
+    def __getitem__(self, indices):
+        data = self.output
+        rank = len(data.get_partial_shape())
+        axes, gather_indices_nodes = [], []
+        slice_axes, slice_starts, slice_ends, slice_steps = [], [], [], []
+        unsqueeze_axes = []
+
+        if not isinstance(indices, tuple):
+            indices = (indices,)
+
+        if any(i is Ellipsis for i in indices):
+            ellipsis_pos = indices.index(Ellipsis)
+            num_specified = sum(
+                i is not Ellipsis and i is not None for i in indices
+            )
+            num_missing = rank - num_specified
+            indices = (
+                indices[:ellipsis_pos]
+                + (builtins.slice(None),) * num_missing
+                + indices[ellipsis_pos + 1 :]
+            )
+
+        def count_unsqueeze_before(dim):
+            return sum(1 for i in range(dim) if indices[i] is None)
+
+        partial_shape = ov_opset.shape_of(data, Type.i32)
+        zero_const = ov_opset.constant(0, Type.i32)
+
+        for dim, index in enumerate(indices):
+            if isinstance(index, bool):
+                raise ValueError(
+                    "OpenVINO backend does not support boolean indexing"
+                )
+            elif isinstance(index, (int, np.integer, np.ndarray)):
+                if isinstance(index, (np.ndarray, np.integer)):
+                    if isinstance(index, np.ndarray) and len(index.shape) != 0:
+                        raise ValueError(
+                            "OpenVINO backend does not support"
+                            "multi-dimensional indexing"
+                        )
+                    index = int(index)
+                actual_dim = dim - count_unsqueeze_before(dim)
+                if not (0 <= actual_dim < rank):
+                    raise IndexError(
+                        f"Index {index} is out of bounds for "
+                        "axis {dim} with rank {rank}"
+                    )
+                length = ov_opset.gather(
+                    partial_shape,
+                    ov_opset.constant([actual_dim], Type.i32),
+                    zero_const,
+                )
+                if index >= 0:
+                    idx_value = ov_opset.constant([index], Type.i32)
+                else:
+                    idx_value = ov_opset.add(
+                        ov_opset.constant([index], Type.i32), length
+                    )
+                axes.append(dim)
+                gather_indices_nodes.append(idx_value.output(0))
+            elif isinstance(index, builtins.slice):
+                if index == builtins.slice(None):
+                    continue
+                if index.step is not None and index.step < 0:
+                    raise ValueError("OpenVINO doesn't support negative steps")
+                slice_axes.append(dim)
+                slice_starts.append(0 if index.start is None else index.start)
+                slice_ends.append(
+                    2**31 - 1 if index.stop is None else index.stop
+                )
+                slice_steps.append(1 if index.step is None else index.step)
+            elif index is None:
+                unsqueeze_axes.append(dim)
+            elif isinstance(index, OpenVINOKerasTensor):
+                index = get_ov_output(index)
+                index_type = index.get_element_type()
+                index_shape = index.get_partial_shape()
+                if index_type == Type.boolean or not index_type.is_integral():
+                    raise ValueError(
+                        "OpenVINO backend does not "
+                        "support {index_type} indexing"
+                    )
+                axes.append(dim)
+                if len(index_shape) > 1:
+                    raise ValueError(
+                        "OpenVINO backend does not "
+                        "support multi-dimensional indexing"
+                    )
+                if len(index_shape) == 0:
+                    index = ov_opset.unsqueeze(index, zero_const).output(0)
+                if index_type != Type.i32:
+                    index = ov_opset.convert(index, Type.i32).output(0)
+                shape_tensor = ov_opset.shape_of(data, Type.i32)
+                axis_i32 = ov_opset.constant([dim], dtype=Type.i32)
+                dim_size = ov_opset.gather(shape_tensor, axis_i32, zero_const)
+                is_negative = ov_opset.less(index, zero_const)
+                adjusted_index = ov_opset.add(index, dim_size)
+                index = ov_opset.select(
+                    is_negative, adjusted_index, index
+                ).output(0)
+                gather_indices_nodes.append(index)
+            else:
+                raise ValueError(
+                    f"Unsupported index type {type(index)} "
+                    "in OpenVINOKerasTensor.__getitem__"
+                )
+
+        if slice_axes:
+            step = ov_opset.constant(slice_steps, Type.i32).output(0)
+            start = ov_opset.constant(slice_starts, Type.i32).output(0)
+            stop = ov_opset.constant(slice_ends, Type.i32).output(0)
+            adjusted_slice_axes = [
+                ax - sum(1 for unsq in unsqueeze_axes if unsq <= ax)
+                for ax in slice_axes
+            ]
+            axes_const = ov_opset.constant(
+                adjusted_slice_axes, Type.i32
+            ).output(0)
+            data = ov_opset.slice(data, start, stop, step, axes_const).output(0)
+
+        if axes:
+            gather_indices_const = (
+                gather_indices_nodes[0]
+                if len(gather_indices_nodes) == 1
+                else ov_opset.concat(gather_indices_nodes, axis=0).output(0)
+            )
+            adjusted_axes = [
+                ax - sum(1 for unsq in unsqueeze_axes if unsq <= ax)
+                for ax in axes
+            ]
+            if len(axes) == 1:
+                data = ov_opset.gather(
+                    data, gather_indices_const, adjusted_axes[0]
+                ).output(0)
+                data = ov_opset.squeeze(data, adjusted_axes[0]).output(0)
+            else:
+                rank = len(data.get_partial_shape())
+                remaining_axes = [
+                    i for i in range(rank) if i not in adjusted_axes
+                ]
+                perm = ov_opset.constant(
+                    adjusted_axes + remaining_axes, Type.i32
+                )
+                data = ov_opset.transpose(data, perm).output(0)
+                data = ov_opset.gather_nd(data, gather_indices_const).output(0)
+
+        if unsqueeze_axes:
+            adjusted_unsqueeze = []
+            for ax in unsqueeze_axes:
+                ax -= sum(1 for s in axes if s < ax)
+                ax -= sum(1 for s in slice_axes if s < ax)
+                adjusted_unsqueeze.append(ax)
+            unsqueeze_const = ov_opset.constant(
+                adjusted_unsqueeze, Type.i32
+            ).output(0)
+            data = ov_opset.unsqueeze(data, unsqueeze_const).output(0)
+
+        return OpenVINOKerasTensor(data)
 
     def __len__(self):
         ov_output = self.output
@@ -292,6 +499,21 @@ class OpenVINOKerasTensor:
         )
         return OpenVINOKerasTensor(ov_opset.mod(first, other).output(0))
 
+    def __array__(self, dtype=None):
+        try:
+            tensor = cast(self, dtype=dtype) if dtype is not None else self
+            return convert_to_numpy(tensor)
+        except Exception as e:
+            raise RuntimeError(
+                "An OpenVINOKerasTensor is symbolic: it's a placeholder "
+                "for a shape and a dtype.\n"
+                "It doesn't have any actual numerical value.\n"
+                "You cannot convert it to a NumPy array."
+            ) from e
+
+    def numpy(self):
+        return self.__array__()
+
 
 def ov_to_keras_type(ov_type):
     for _keras_type, _ov_type in OPENVINO_DTYPES.items():
@@ -392,52 +614,46 @@ def _is_scalar(elem):
     return not isinstance(elem, (list, tuple, set, dict))
 
 
-def _get_first_element(x):
-    if isinstance(x, (tuple, list)):
-        for elem_in_x in x:
-            elem = _get_first_element(elem_in_x)
-            if elem is not None:
-                return elem
-    elif _is_scalar(x):
-        return x
-    return None
-
-
 def convert_to_tensor(x, dtype=None, sparse=None, ragged=None):
     if sparse:
         raise ValueError("`sparse=True` is not supported with openvino backend")
     if ragged:
         raise ValueError("`ragged=True` is not supported with openvino backend")
+    if dtype is not None:
+        dtype = standardize_dtype(dtype)
     if isinstance(x, OpenVINOKerasTensor):
+        if dtype and dtype != standardize_dtype(x.dtype):
+            x = cast(x, dtype)
         return x
     elif isinstance(x, np.ndarray):
         if dtype is not None:
-            dtype = standardize_dtype(dtype)
             ov_type = OPENVINO_DTYPES[dtype]
-            return OpenVINOKerasTensor(ov_opset.constant(x, ov_type).output(0))
-        return OpenVINOKerasTensor(ov_opset.constant(x).output(0))
-    elif isinstance(x, (list, tuple)):
-        if dtype is not None:
-            dtype = standardize_dtype(dtype)
         else:
-            # try to properly deduce element type
-            elem = _get_first_element(x)
-            if isinstance(elem, float):
-                dtype = "float32"
-            elif isinstance(elem, int):
-                dtype = "int32"
+            ov_type = OPENVINO_DTYPES[standardize_dtype(x.dtype)]
+        return OpenVINOKerasTensor(ov_opset.constant(x, ov_type).output(0))
+    elif isinstance(x, (list, tuple)):
+        if dtype is None:
+            dtype = result_type(
+                *[
+                    getattr(item, "dtype", type(item))
+                    for item in tree.flatten(x)
+                ]
+            )
         x = np.array(x, dtype=dtype)
-        return OpenVINOKerasTensor(ov_opset.constant(x).output(0), x)
-    elif isinstance(x, (float, int)):
-        dtype = standardize_dtype(dtype)
         ov_type = OPENVINO_DTYPES[dtype]
         return OpenVINOKerasTensor(ov_opset.constant(x, ov_type).output(0), x)
-    if dtype is not None:
-        dtype = standardize_dtype(dtype)
+    elif isinstance(x, (float, int, bool)):
+        if dtype is None:
+            dtype = standardize_dtype(type(x))
+        ov_type = OPENVINO_DTYPES[dtype]
+        return OpenVINOKerasTensor(ov_opset.constant(x, ov_type).output(0), x)
+    elif isinstance(x, ov.Output):
+        return OpenVINOKerasTensor(x)
     if isinstance(x, Variable):
+        x = x.value
         if dtype and dtype != x.dtype:
-            return x.value.astype(dtype)
-        return x.value
+            x = cast(x, dtype)
+        return x
     if not is_tensor(x) and standardize_dtype(dtype) == "bfloat16":
         return ov.Tensor(np.asarray(x).astype(dtype))
     if dtype is None:
@@ -478,8 +694,10 @@ def convert_to_numpy(x):
         ov_model = Model(results=[ov_result], parameters=[])
         ov_compiled_model = compile_model(ov_model, get_device())
         result = ov_compiled_model({})[0]
-    except:
-        raise "`convert_to_numpy` cannot convert to numpy"
+    except Exception as inner_exception:
+        raise RuntimeError(
+            "`convert_to_numpy` failed to convert the tensor."
+        ) from inner_exception
     return result
 
 
@@ -496,6 +714,7 @@ def shape(x):
 
 
 def cast(x, dtype):
+    dtype = standardize_dtype(dtype)
     ov_type = OPENVINO_DTYPES[dtype]
     x = get_ov_output(x)
     return OpenVINOKerasTensor(ov_opset.convert(x, ov_type).output(0))
@@ -565,33 +784,57 @@ def scatter_update(inputs, indices, updates):
     )
 
 
-def slice(inputs, start_indices, lengths):
+def slice(inputs, start_indices, shape):
     inputs = get_ov_output(inputs)
+    if isinstance(start_indices, (list, np.ndarray)):
+        start_indices = tuple(start_indices)
+    if isinstance(shape, (list, np.ndarray)):
+        shape = tuple(shape)
     assert isinstance(start_indices, tuple), (
         "`slice` is not supported by openvino backend"
-        " for `start_indices` of type {}".format(type(lengths))
+        " for `start_indices` of type {}".format(type(start_indices))
     )
-    assert isinstance(lengths, tuple), (
+    assert isinstance(shape, tuple), (
         "`slice` is not supported by openvino backend"
-        " for `lengths` of type {}".format(type(lengths))
+        " for `shape` of type {}".format(type(shape))
     )
 
     axes = []
     start = []
     stop = []
-    for idx, length in enumerate(lengths):
+
+    def prepare_slice_index(val):
+        val_type = val.get_element_type()
+        if not val_type.is_integral():
+            raise ValueError(
+                "`slice` is not supported by OpenVINO backend "
+                "for `start_indices` or `shape` with non-integer types"
+            )
+        if val_type != Type.i32:
+            val = ov_opset.convert(val, Type.i32).output(0)
+        if len(val.get_partial_shape()) == 0:
+            val = ov_opset.unsqueeze(
+                val, ov_opset.constant(0, Type.i32)
+            ).output(0)
+        return val
+
+    for idx, length in enumerate(shape):
         if length is not None and length >= 0:
             axes.append(idx)
-            start.append(start_indices[idx])
-            stop.append(start_indices[idx] + length)
+            start_val = prepare_slice_index(get_ov_output(start_indices[idx]))
+            stop_val = prepare_slice_index(
+                get_ov_output(start_indices[idx] + length)
+            )
+            start.append(start_val)
+            stop.append(stop_val)
 
     if len(axes) == 0:
         return inputs
 
     step = [1] * len(start)
     step = ov_opset.constant(step, Type.i32).output(0)
-    start = ov_opset.constant(start, Type.i32).output(0)
-    stop = ov_opset.constant(stop, Type.i32).output(0)
+    start = ov_opset.concat(start, axis=0).output(0)
+    stop = ov_opset.concat(stop, axis=0).output(0)
     axes = ov_opset.constant(axes, Type.i32).output(0)
     return OpenVINOKerasTensor(
         ov_opset.slice(inputs, start, stop, step, axes).output(0)
@@ -610,10 +853,111 @@ def while_loop(
     loop_vars,
     maximum_iterations=None,
 ):
-    raise NotImplementedError(
-        "`while_loop` is not supported with openvino backend"
+    def flatten_structure(data):
+        if isinstance(data, dict):
+            return [v for k in sorted(data) for v in flatten_structure(data[k])]
+        elif isinstance(data, (tuple, list)):
+            return [v for item in data for v in flatten_structure(item)]
+        else:
+            return [data]
+
+    def pack_structure(template, flat):
+        if isinstance(template, dict):
+            keys = sorted(template)
+            packed = {}
+            for k in keys:
+                value, flat = pack_structure(template[k], flat)
+                packed[k] = value
+            return packed, flat
+        elif isinstance(template, (tuple, list)):
+            packed = []
+            for item in template:
+                value, flat = pack_structure(item, flat)
+                packed.append(value)
+            return (
+                tuple(packed) if isinstance(template, tuple) else packed
+            ), flat
+        else:
+            return flat[0], flat[1:]
+
+    is_scalar_input = _is_scalar(loop_vars)
+
+    if is_scalar_input:
+        loop_vars = (loop_vars,)
+    elif isinstance(loop_vars, (list, np.ndarray)):
+        loop_vars = tuple(loop_vars)
+    else:
+        assert isinstance(loop_vars, (tuple, dict)), (
+            f"Unsupported type {type(loop_vars)} for loop_vars"
+        )
+
+    flat_loop_vars = flatten_structure(loop_vars)
+    loop_vars_ov = [get_ov_output(var) for var in flat_loop_vars]
+
+    maximum_iterations = (
+        ov_opset.constant(-1, Type.i32).output(0)
+        if maximum_iterations is None
+        else get_ov_output(maximum_iterations)
     )
 
+    trip_count = maximum_iterations
+    execution_condition = ov_opset.constant(True, Type.boolean).output(0)
+    loop = ov_opset.loop(trip_count, execution_condition)
+
+    shapes = [var.get_partial_shape() for var in loop_vars_ov]
+    types = [var.get_element_type() for var in loop_vars_ov]
+    params = [
+        ov_opset.parameter(shape, dtype) for shape, dtype in zip(shapes, types)
+    ]
+    param_tensors = [OpenVINOKerasTensor(p.output(0)) for p in params]
+
+    packed_args, _ = pack_structure(loop_vars, param_tensors)
+    if isinstance(packed_args, dict):
+        body_out = body(packed_args)
+    else:
+        body_out = body(*packed_args)
+
+    if not isinstance(body_out, (list, tuple, dict)):
+        body_out = (body_out,)
+
+    flat_body_out = flatten_structure(body_out)
+    if isinstance(packed_args, dict):
+        cond_output = get_ov_output(cond(body_out))
+    else:
+        cond_output = get_ov_output(cond(*body_out))
+
+    if len(cond_output.get_partial_shape()) != 0:
+        raise ValueError(
+            "`cond` function must return a scalar boolean value, "
+            "but got shape {}".format(cond_output.get_partial_shape())
+        )
+
+    for p, out in zip(params, flat_body_out):
+        out_shape = get_ov_output(out).get_partial_shape()
+        p.set_partial_shape(out_shape)
+
+    results = [cond_output] + [get_ov_output(x) for x in flat_body_out]
+    body_func = Model(results=results, parameters=params)
+    loop.set_function(body_func)
+    loop.set_special_body_ports([-1, 0])
+
+    for param, init_val, next_val in zip(params, loop_vars_ov, flat_body_out):
+        loop.set_merged_input(param, init_val, get_ov_output(next_val))
+
+    outputs_flat = [
+        OpenVINOKerasTensor(loop.get_iter_value(get_ov_output(val)))
+        for val in flat_body_out
+    ]
+    final_output, _ = pack_structure(loop_vars, outputs_flat)
+
+    if is_scalar_input:
+        if isinstance(final_output, tuple):
+            return final_output[0]
+        else:
+            return final_output
+    else:
+        return final_output
+
 
 def fori_loop(lower, upper, body_fun, init_val):
     raise NotImplementedError(
@@ -621,8 +965,8 @@ def fori_loop(lower, upper, body_fun, init_val):
     )
 
 
-def stop_gradient(x):
-    return x
+def stop_gradient(variable):
+    return variable
 
 
 def unstack(x, num=None, axis=0):
