@@ -74,10 +74,9 @@ class TensorBoard(Callback):
             Batch-level summary writing is also available via `train_step`
             override. Please see
             [TensorBoard Scalars tutorial](
-                https://www.tensorflow.org/tensorboard/scalars_and_keras#batch-level_logging)  # noqa: E501
+                https://www.tensorflow.org/tensorboard/scalars_and_keras#batch-level_logging)
             for more details.
-        profile_batch: (Not supported at this time)
-            Profile the batch(es) to sample compute characteristics.
+        profile_batch: Profile the batch(es) to sample compute characteristics.
             profile_batch must be a non-negative integer or a tuple of integers.
             A pair of positive integers signify a range of batches to profile.
             By default, profiling is disabled.
@@ -152,7 +151,7 @@ class TensorBoard(Callback):
         log_dir='./logs', profile_batch=(10,20))
     model.fit(x_train, y_train, epochs=2, callbacks=[tensorboard_callback])
     ```
-    """
+    """  # noqa: E501
 
     def __init__(
         self,
@@ -176,16 +175,26 @@ class TensorBoard(Callback):
         self.update_freq = 1 if update_freq == "batch" else update_freq
         self.embeddings_freq = embeddings_freq
         self.embeddings_metadata = embeddings_metadata
-        if profile_batch and backend.backend() != "tensorflow":
-            # TODO: profiling not available in JAX/torch
-            raise ValueError(
-                "Profiling is not yet available with the "
-                f"{backend.backend()} backend. Please open a PR "
-                "if you'd like to add this feature. Received: "
-                f"profile_batch={profile_batch} (must be 0)"
-            )
+        if profile_batch:
+            if backend.backend() not in ("jax", "tensorflow"):
+                # TODO: profiling not available in torch, numpy
+                raise ValueError(
+                    "Profiling is not yet available with the "
+                    f"{backend.backend()} backend. Please open a PR "
+                    "if you'd like to add this feature. Received: "
+                    f"profile_batch={profile_batch} (must be 0)"
+                )
+            elif backend.backend() == "jax":
+                if sys.version_info[1] < 12:
+                    warnings.warn(
+                        "Profiling with the "
+                        f"{backend.backend()} backend requires python >= 3.12."
+                    )
+                    profile_batch = 0
+
         self._init_profile_batch(profile_batch)
         self._global_train_batch = 0
+        self._global_test_batch = 0
         self._previous_epoch_iterations = 0
         self._train_accumulated_time = 0
         self._batch_start_time = 0
@@ -204,11 +213,7 @@ class TensorBoard(Callback):
         self._log_write_dir = self.log_dir
 
         self._train_dir = os.path.join(self._log_write_dir, "train")
-        self._train_step = 0
-
         self._val_dir = os.path.join(self._log_write_dir, "validation")
-        self._val_step = 0
-
         self._writers = {}  # Resets writers.
 
         self._should_write_train_graph = False
@@ -384,6 +389,8 @@ class TensorBoard(Callback):
         # We track the status here to make sure callbacks do not interfere with
         # each other. The callback will only stop the profiler it started.
         self._profiler_started = False
+        self._batch_trace_context = None
+
         if self._start_batch > 0:
             # Warm up and improve the profiling accuracy.
             self._start_profiler(logdir="")
@@ -399,7 +406,7 @@ class TensorBoard(Callback):
     def on_train_begin(self, logs=None):
         self._global_train_batch = 0
         self._previous_epoch_iterations = 0
-        self._push_writer(self._train_writer, self._train_step)
+        self._push_writer(self._train_writer, self._global_train_batch)
 
     def on_train_end(self, logs=None):
         self._pop_writer()
@@ -410,7 +417,7 @@ class TensorBoard(Callback):
         self._close_writers()
 
     def on_test_begin(self, logs=None):
-        self._push_writer(self._val_writer, self._val_step)
+        self._push_writer(self._val_writer, self._global_test_batch)
 
     def on_test_end(self, logs=None):
         if self.model.optimizer and hasattr(self.model.optimizer, "iterations"):
@@ -423,11 +430,6 @@ class TensorBoard(Callback):
                     )
         self._pop_writer()
 
-    def _implements_train_batch_hooks(self):
-        # Only call batch hooks when tracing or write_steps_per_second are
-        # enabled
-        return self._should_trace or self.write_steps_per_second
-
     def on_train_batch_begin(self, batch, logs=None):
         self._global_train_batch += 1
         if self.write_steps_per_second:
@@ -437,6 +439,10 @@ class TensorBoard(Callback):
 
         if self._global_train_batch == self._start_batch:
             self._start_trace()
+        if self._profiler_started:
+            self._batch_trace_context = backend.tensorboard.start_batch_trace(
+                batch
+            )
 
     def on_train_batch_end(self, batch, logs=None):
         if self._should_write_train_graph:
@@ -447,21 +453,28 @@ class TensorBoard(Callback):
             self.summary.scalar(
                 "batch_steps_per_second",
                 1.0 / batch_run_time,
-                step=self._train_step,
+                step=self._global_train_batch,
             )
 
         # `logs` isn't necessarily always a dict
         if isinstance(logs, dict):
             for name, value in logs.items():
                 self.summary.scalar(
-                    "batch_" + name, value, step=self._train_step
+                    "batch_" + name, value, step=self._global_train_batch
                 )
 
         if not self._should_trace:
             return
 
-        if self._is_tracing and self._global_train_batch >= self._stop_batch:
-            self._stop_trace()
+        if self._is_tracing:
+            if self._profiler_started and self._batch_trace_context is not None:
+                backend.tensorboard.stop_batch_trace(self._batch_trace_context)
+                self._batch_trace_context = None
+            if self._global_train_batch >= self._stop_batch:
+                self._stop_trace()
+
+    def on_test_batch_begin(self, batch, logs=None):
+        self._global_test_batch += 1
 
     def on_epoch_begin(self, epoch, logs=None):
         # Keeps track of epoch for profiling.
@@ -483,7 +496,7 @@ class TensorBoard(Callback):
 
     def _start_trace(self):
         self.summary.trace_on(graph=True, profiler=False)
-        self._start_profiler(logdir=self.log_dir)
+        self._start_profiler(logdir=self._train_dir)
         self._is_tracing = True
 
     def _stop_trace(self, batch=None):
