@@ -58,17 +58,18 @@ def load_data(label_mode="fine"):
             f"Received: label_mode={label_mode}."
         )
 
-    dirname = "cifar-100-python"
+    dirname = "cifar-100-python-target"
     origin = "https://www.cs.toronto.edu/~kriz/cifar-100-python.tar.gz"
     path = get_file(
         fname=dirname,
         origin=origin,
-        untar=True,
+        extract=True,
         file_hash=(  # noqa: E501
             "85cd44d02ba6437773c5bbd22e183051d648de2e7d6b014e1ef29b855ba677a7"
         ),
     )
 
+    path = os.path.join(path, "cifar-100-python")
     fpath = os.path.join(path, "train")
     x_train, y_train = load_batch(fpath, label_key=label_mode + "_labels")
 
