@@ -61,7 +61,8 @@ class ReLU(Layer):
         self.negative_slope = negative_slope
         self.threshold = threshold
         self.supports_masking = True
-        self.built = True
+
+        self._build_at_init()
 
     def call(self, inputs):
         return activations.relu(
