@@ -1,4 +1,5 @@
-from keras.src import backend
+import warnings
+
 from keras.src import constraints
 from keras.src import initializers
 from keras.src import ops
@@ -83,12 +84,6 @@ class LayerNormalization(Layer):
             When the next layer is linear (also e.g. `nn.relu`), this can be
             disabled since the scaling will be done by the next layer.
             Defaults to `True`.
-        rms_scaling: If True, `center` and `scale` are ignored, and the
-            inputs are scaled by `gamma` and the inverse square root
-            of the square of all inputs. This is an approximate and faster
-            approach that avoids ever computing the mean of the input. Note that
-            this *isn't* equivalent to the computation that the
-            `keras.layers.RMSNormalization` layer performs.
         beta_initializer: Initializer for the beta weight. Defaults to zeros.
         gamma_initializer: Initializer for the gamma weight. Defaults to ones.
         beta_regularizer: Optional regularizer for the beta weight.
@@ -113,7 +108,6 @@ class LayerNormalization(Layer):
         epsilon=1e-3,
         center=True,
         scale=True,
-        rms_scaling=False,
         beta_initializer="zeros",
         gamma_initializer="ones",
         beta_regularizer=None,
@@ -122,6 +116,15 @@ class LayerNormalization(Layer):
         gamma_constraint=None,
         **kwargs,
     ):
+        rms_scaling = kwargs.pop("rms_scaling", False)
+        if rms_scaling:
+            warnings.warn(
+                "You passed `rms_scaling=True`, which is deprecated. This "
+                "argument incorrectly scales the input by the variance, not "
+                "the root mean square. To correctly use RMS Normalization, "
+                "please use `keras.layers.RMSNormalization` instead."
+            )
+
         super().__init__(**kwargs)
         if isinstance(axis, (list, tuple)):
             self.axis = list(axis)
@@ -180,56 +183,14 @@ class LayerNormalization(Layer):
             self.beta = None
 
     def call(self, inputs):
-        # Compute the axes along which to reduce the mean / variance
-        input_shape = inputs.shape
-        ndims = len(input_shape)
-
-        # Broadcasting only necessary for norm when the axis is not just
-        # the last dimension
-        broadcast_shape = [1] * ndims
-        for dim in self.axis:
-            broadcast_shape[dim] = input_shape[dim]
-
-        def _broadcast(v):
-            if (
-                v is not None
-                and len(v.shape) != ndims
-                and self.axis != [ndims - 1]
-            ):
-                return ops.reshape(v, broadcast_shape)
-            return v
-
-        compute_dtype = backend.result_type(inputs.dtype, "float32")
-        # LN is prone to overflow with float16/bfloat16 inputs, so we upcast to
-        # float32 for the subsequent computations.
-        inputs = ops.cast(inputs, compute_dtype)
-
-        if self.rms_scaling:
-            # Calculate outputs with only variance and gamma if rms scaling
-            # is enabled
-            # Calculate the variance along self.axis (layer activations).
-            variance = ops.var(inputs, axis=self.axis, keepdims=True)
-            inv = ops.rsqrt(variance + self.epsilon)
-
-            outputs = (
-                inputs * inv * ops.cast(_broadcast(self.gamma), inputs.dtype)
-            )
-        else:
-            # Calculate the mean & variance along self.axis (layer activations).
-            mean, variance = ops.moments(inputs, axes=self.axis, keepdims=True)
-            gamma, beta = _broadcast(self.gamma), _broadcast(self.beta)
-
-            inv = ops.rsqrt(variance + self.epsilon)
-            if gamma is not None:
-                gamma = ops.cast(gamma, inputs.dtype)
-                inv = inv * gamma
-
-            res = -mean * inv
-            if beta is not None:
-                beta = ops.cast(beta, inputs.dtype)
-                res = res + beta
-
-            outputs = inputs * inv + res
+        outputs = ops.layer_normalization(
+            inputs,
+            self.gamma,
+            self.beta,
+            self.axis,
+            self.epsilon,
+            rms_scaling=self.rms_scaling,
+        )
         return ops.cast(outputs, self.compute_dtype)
 
     def compute_output_shape(self, input_shape):
