@@ -6,12 +6,13 @@ from keras.src.layers.layer import Layer
 from keras.src.layers.preprocessing.tf_data_layer import TFDataLayer
 from keras.src.saving import saving_lib
 from keras.src.saving import serialization_lib
+from keras.src.saving.keras_saveable import KerasSaveable
 from keras.src.utils import backend_utils
 from keras.src.utils.module_utils import tensorflow as tf
 from keras.src.utils.naming import auto_name
 
 
-class Cross:
+class Cross(KerasSaveable):
     def __init__(self, feature_names, crossing_dim, output_mode="one_hot"):
         if output_mode not in {"int", "one_hot"}:
             raise ValueError(
@@ -23,6 +24,9 @@ class Cross:
         self.crossing_dim = crossing_dim
         self.output_mode = output_mode
 
+    def _obj_type(self):
+        return "Cross"
+
     @property
     def name(self):
         return "_X_".join(self.feature_names)
@@ -39,7 +43,7 @@ class Cross:
         return cls(**config)
 
 
-class Feature:
+class Feature(KerasSaveable):
     def __init__(self, dtype, preprocessor, output_mode):
         if output_mode not in {"int", "one_hot", "float"}:
             raise ValueError(
@@ -55,6 +59,9 @@ class Feature:
         self.preprocessor = preprocessor
         self.output_mode = output_mode
 
+    def _obj_type(self):
+        return "Feature"
+
     def get_config(self):
         return {
             "dtype": self.dtype,
