@@ -148,7 +148,6 @@ class RNN(Layer):
                 shape=(self.units, self.units),
                 initializer='uniform',
                 name='recurrent_kernel')
-            self.built = True
 
         def call(self, inputs, states):
             prev_output = states[0]
@@ -284,7 +283,6 @@ class RNN(Layer):
                         f"batch size: sequence.shape={sequences_shape}"
                     )
                 self._create_state_variables(sequences_shape[0])
-        self.built = True
 
     @tracking.no_automatic_dependency_tracking
     def _create_state_variables(self, batch_size):
@@ -323,7 +321,7 @@ class RNN(Layer):
     def reset_state(self):
         if self.states is not None:
             for v in self.states:
-                v.assign(ops.zeros_like(v))
+                v.assign(ops.zeros_like(v.value))
 
     def inner_loop(self, sequences, initial_state, mask, training=False):
         cell_kwargs = {}
