@@ -249,7 +249,7 @@ class Trainer:
     @property
     def metrics(self):
         # Order: loss tracker, individual loss trackers, compiled metrics,
-        # custom metrcis, sublayer metrics.
+        # custom metrics, sublayer metrics.
         metrics = []
         if self.compiled:
             if self._loss_tracker is not None:
@@ -1072,7 +1072,7 @@ class Trainer:
                 )
 
             if data_batch is None:
-                for _, data_or_iterator in iterator:
+                for _, _, data_or_iterator in iterator:
                     if isinstance(data_or_iterator, (list, tuple)):
                         data_batch = data_or_iterator[0]
                     else:
