@@ -40,8 +40,10 @@ def check_graphviz():
 
 
 def add_edge(dot, src, dst):
-    if not dot.get_edge(src, dst):
-        edge = pydot.Edge(src, dst)
+    src_id = str(id(src))
+    dst_id = str(id(dst))
+    if not dot.get_edge(src_id, dst_id):
+        edge = pydot.Edge(src_id, dst_id)
         edge.set("penwidth", "2")
         dot.add_edge(edge)
 
@@ -189,14 +191,6 @@ def make_node(layer, **kwargs):
     return node
 
 
-def remove_unused_edges(dot):
-    nodes = [v.get_name() for v in dot.get_nodes()]
-    for edge in dot.get_edges():
-        if edge.get_destination() not in nodes:
-            dot.del_edge(edge.get_source(), edge.get_destination())
-    return dot
-
-
 @keras_export("keras.utils.model_to_dot")
 def model_to_dot(
     model,
@@ -290,11 +284,11 @@ def model_to_dot(
         layers = model._operations
 
     # Create graph nodes.
-    sub_n_first_node = {}
-    sub_n_last_node = {}
     for i, layer in enumerate(layers):
-        # Process nested functional models.
-        if expand_nested and isinstance(layer, functional.Functional):
+        # Process nested functional and sequential models.
+        if expand_nested and isinstance(
+            layer, (functional.Functional, sequential.Sequential)
+        ):
             submodel = model_to_dot(
                 layer,
                 show_shapes,
@@ -306,10 +300,6 @@ def model_to_dot(
                 show_layer_activations=show_layer_activations,
                 show_trainable=show_trainable,
             )
-            # sub_n : submodel
-            sub_n_nodes = submodel.get_nodes()
-            sub_n_first_node[layer.name] = sub_n_nodes[0]
-            sub_n_last_node[layer.name] = sub_n_nodes[-1]
             dot.add_subgraph(submodel)
 
         else:
@@ -317,54 +307,98 @@ def model_to_dot(
             dot.add_node(node)
 
     # Connect nodes with edges.
-    # Sequential case.
     if isinstance(model, sequential.Sequential):
-        for i in range(len(layers) - 1):
-            inbound_layer_id = str(id(layers[i]))
-            layer_id = str(id(layers[i + 1]))
-            add_edge(dot, inbound_layer_id, layer_id)
-        return dot
+        if not expand_nested:
+            # Single Sequential case.
+            for i in range(len(layers) - 1):
+                add_edge(dot, layers[i], layers[i + 1])
+            return dot
+        else:
+            # The first layer is connected to the `InputLayer`, which is not
+            # represented for Sequential models, so we skip it. What will draw
+            # the incoming edge from outside of the sequential model is the
+            # edge connecting the Sequential model itself.
+            layers = model.layers[1:]
 
-    # Functional case.
-    for i, layer in enumerate(layers):
-        layer_id = str(id(layer))
-        for i, node in enumerate(layer._inbound_nodes):
-            node_key = make_node_key(layer, i)
-            if node_key in model._nodes:
-                for parent_node in node.parent_nodes:
-                    inbound_layer = parent_node.operation
-                    inbound_layer_id = str(id(inbound_layer))
-                    if not expand_nested:
-                        assert dot.get_node(inbound_layer_id)
-                        assert dot.get_node(layer_id)
-                        add_edge(dot, inbound_layer_id, layer_id)
+    # Functional and nested Sequential case.
+    for layer in layers:
+        # Go from current layer to input `Node`s.
+        for inbound_index, inbound_node in enumerate(layer._inbound_nodes):
+            # `inbound_node` is a `Node`.
+            if (
+                isinstance(model, functional.Functional)
+                and make_node_key(layer, inbound_index) not in model._nodes
+            ):
+                continue
+
+            # Go from input `Node` to `KerasTensor` representing that input.
+            for input_index, input_tensor in enumerate(
+                inbound_node.input_tensors
+            ):
+                # `input_tensor` is a `KerasTensor`.
+                # `input_history` is a `KerasHistory`.
+                input_history = input_tensor._keras_history
+                if input_history.operation is None:
+                    # Operation is `None` for `Input` tensors.
+                    continue
+
+                # Go from input `KerasTensor` to the `Operation` that produced
+                # it as an output.
+                input_node = input_history.operation._inbound_nodes[
+                    input_history.node_index
+                ]
+                output_index = input_history.tensor_index
+
+                # Tentative source and destination of the edge.
+                source = input_node.operation
+                destination = layer
+
+                if not expand_nested:
+                    # No nesting, connect directly.
+                    add_edge(dot, source, layer)
+                    continue
+
+                # ==== Potentially nested models case ====
+
+                # ---- Resolve the source of the edge ----
+                while isinstance(
+                    source,
+                    (functional.Functional, sequential.Sequential),
+                ):
+                    # When `source` is a `Functional` or `Sequential` model, we
+                    # need to connect to the correct box within that model.
+                    # Functional and sequential models do not have explicit
+                    # "output" boxes, so we need to find the correct layer that
+                    # produces the output we're connecting to, which can be
+                    # nested several levels deep in sub-models. Hence the while
+                    # loop to continue going into nested models until we
+                    # encounter a real layer that's not a `Functional` or
+                    # `Sequential`.
+                    source, _, output_index = source.outputs[
+                        output_index
+                    ]._keras_history
+
+                # ---- Resolve the destination of the edge ----
+                while isinstance(
+                    destination,
+                    (functional.Functional, sequential.Sequential),
+                ):
+                    if isinstance(destination, functional.Functional):
+                        # When `destination` is a `Functional`, we point to the
+                        # specific `InputLayer` in the model.
+                        destination = destination.inputs[
+                            input_index
+                        ]._keras_history.operation
                     else:
-                        # if inbound_layer is not Functional
-                        if not isinstance(inbound_layer, functional.Functional):
-                            # if current layer is not Functional
-                            if not isinstance(layer, functional.Functional):
-                                assert dot.get_node(inbound_layer_id)
-                                assert dot.get_node(layer_id)
-                                add_edge(dot, inbound_layer_id, layer_id)
-                            # if current layer is Functional
-                            elif isinstance(layer, functional.Functional):
-                                add_edge(
-                                    dot,
-                                    inbound_layer_id,
-                                    sub_n_first_node[layer.name].get_name(),
-                                )
-                        # if inbound_layer is Functional
-                        elif isinstance(inbound_layer, functional.Functional):
-                            name = sub_n_last_node[
-                                inbound_layer.name
-                            ].get_name()
-                            if isinstance(layer, functional.Functional):
-                                output_name = sub_n_first_node[
-                                    layer.name
-                                ].get_name()
-                                add_edge(dot, name, output_name)
-                            else:
-                                add_edge(dot, name, layer_id)
+                        # When `destination` is a `Sequential`, there is no
+                        # explicit "input" box, so we want to point to the first
+                        # box in the model, but it may itself be another model.
+                        # Hence the while loop to continue going into nested
+                        # models until we encounter a real layer that's not a
+                        # `Functional` or `Sequential`.
+                        destination = destination.layers[0]
+
+                add_edge(dot, source, destination)
     return dot
 
 
@@ -467,7 +501,6 @@ def plot_model(
     to_file = str(to_file)
     if dot is None:
         return
-    dot = remove_unused_edges(dot)
     _, extension = os.path.splitext(to_file)
     if not extension:
         extension = "png"
